@@ -9,7 +9,7 @@
  
 #ifndef HIDE_SOURCE_STRINGS
 static const char cvsid[] = 
-	"$Id: mbus_engine.c,v 1.216 2001/03/28 14:31:56 ucacoxh Exp $";
+	"$Id: mbus_engine.c,v 1.218 2002/05/21 21:29:50 ucaccsp Exp $";
 #endif /* HIDE_SOURCE_STRINGS */
 
 #include "config_unix.h"
@@ -604,11 +604,11 @@ static void rx_tool_rat_logstats(char *srce, char *args, session_t *sp)
 #endif
 			gettimeofday(&t, NULL);
 			gethostname(hname, 64);
-			sprintf(fname, "rat-%p-%ld.%06ld-%s-%s.log", sp, t.tv_sec, t.tv_usec, hname, uname);
+			sprintf(fname, "rat-%p-%ld.%06ld-%s-%s.log", sp, t.tv_sec, (long)t.tv_usec, hname, uname);
 			cname = rtp_get_sdes(sp->rtp_session[0], rtp_my_ssrc(sp->rtp_session[0]), RTCP_SDES_CNAME);
 
 			sp->logger = fopen(fname, "w");
-			fprintf(sp->logger, "tool_start %lu.%06lu ", t.tv_sec + SECS_BETWEEN_1900_1970, t.tv_usec);
+			fprintf(sp->logger, "tool_start %lu.%06lu ", t.tv_sec + SECS_BETWEEN_1900_1970, (long)t.tv_usec);
 			fprintf(sp->logger, "0x%08lx\n", (unsigned long) rtp_my_ssrc(sp->rtp_session[0]));
 		} else {
 			fclose(sp->logger);
@@ -731,7 +731,7 @@ static void rx_audio_file_rec_open(char *srce, char *args, session_t *sp)
                 if (sp->out_file) snd_write_close(&sp->out_file);
 
                 sf_fmt.encoding    = SNDFILE_ENCODING_L16;
-                sf_fmt.sample_rate = (uint16_t)ofmt->sample_rate;
+                sf_fmt.sample_rate = (uint32_t)ofmt->sample_rate;
                 sf_fmt.channels    = (uint16_t)ofmt->channels;
 #ifdef WIN32
                 if (snd_write_open(&sp->out_file, file, "wav", &sf_fmt)) {
@@ -1275,7 +1275,7 @@ set_red_parameters(session_t *sp, char *sec_enc, int offset)
 
         pri_id = codec_get_by_payload(sp->encodings[0]);
         pcf    = codec_get_format(pri_id);
-        red_id = codec_get_matching(sec_enc, (uint16_t)pcf->format.sample_rate, (uint16_t)pcf->format.channels);
+        red_id = codec_get_matching(sec_enc, (uint32_t)pcf->format.sample_rate, (uint16_t)pcf->format.channels);
         if (!codec_id_is_valid(red_id)) {
                 debug_msg("Failed to get redundant codec requested (%s)\n", sec_enc);
                 red_id = pri_id;  /* Use same as primary */
